@@ -6,19 +6,32 @@ use warnings;
 
 use DBI;
 use Error qw(:try);
-use TheSchwartz;
-use TheSchwartz::Job;
+use Helios::TheSchwartz;
+use Helios::ObjectDriver::DBI;
+
 require XML::Simple;
-$XML::Simple::PREFERRED_PARSER = 'XML::Parser';
 
 use Helios::Error;
 use Helios::JobHistory;
 
-our $VERSION = '2.221_363';
+our $VERSION = '2.601_3610';
 
 our $D_OD_RETRIES = 3;
 our $D_OD_RETRY_INTERVAL = 5;
 
+# 2011-12-15:  Removed setting $XML::Simple::PREFERRED_PARSER.
+# 2012-01-01:  Changed failed() and failedNoRetry() methods to truncate error 
+# string at 256 chars.  That's the max length of the matching field in the 
+# ERROR table.  Updated copyright info.
+# 2012-03-27:  Documented accessor methods and greatly expanded and updated 
+# JOB SUBMISSION documentation.
+# 2012-04-01:  Added setDriver() and initDriver() methods.  Refactored old 
+# getDriver() into initDriver(), and changed getDriver() to call initDriver().
+# 2012-04-25:  Added deferred() method.
+# [LH] 2012-07-11: submit(): changed to use Helios::TheSchwartz instead of 
+# base TheSchwartz to implement database connection caching.
+
+
 =head1 NAME
 
 Helios::Job - base class for jobs in the Helios job processing system
@@ -29,9 +42,35 @@ Helios::Job is the standard representation of jobs in the Helios framework.  It
 related to the underlying TheSchwartz::Job objects, and provides its own methods for manipulating 
 jobs in the Helios system.
 
-
 =head1 ACCESSOR METHODS
 
+These accessors allow access to information about an instantiated Helios::Job 
+object:
+
+ debug()          whether Debug Mode is enabled or not
+ get/setConfig()  Helios configuration passed by the system to the job object
+ get/setArgs()    hashref of the job's arguments (interpreted from the arg XML)
+ get/setArgXML()  the raw XML of the job arguments
+
+Several accessors are pass-through accessors to access values in the 
+underlying TheSchwartz::Job object
+ 
+ get/setJobid()         jobid of the job in the job queue
+ get/setFailures()      number of previous failures of the job before current run
+ get/setFuncid()        funcid value of the job (maps to funcname in Helios db)
+ get/setFuncname()      funcname value of the job (maps to funcid in Helios db)
+ get/setUniqkey()       uniqkey value of the job (see TheSchwartz documentation)
+ get/setRunAfter()      current run_after value of the job
+ get/setGrabbedUntil()  current grabbed_until value of the job
+ get/setCoalesce()      coalesce value of the job (see TheSchwartz documentation)
+
+When running a job, your service class need not access any of these values 
+directly, though the information is available if you need it (for example, 
+to log how many failures your job has encountered before the current run).  
+When submitting a job, several of the set* accessors are needed to set up the
+job before submission; see the section on the submit() method for more 
+information.
+
 =cut
 
 sub setConfig { $_[0]->{config} = $_[1]; }
@@ -64,15 +103,25 @@ sub getGrabbedUntil { return $_[0]->job()->grabbed_until; }
 sub setCoalesce { return $_[0]->job()->coalesce($_[1]); }
 sub getCoalesce { return $_[0]->job()->coalesce; }
 
+# BEGIN CODE Copyright (C) 2012 by Andrew Johnson.
+sub setDriver { $_[0]->{driver} = $_[1]; }
+sub getDriver { 
+	if ( defined($_[0]->{driver}) ) {
+		return $_[0]->{driver};
+	} else {
+		return $_[0]->initDriver();
+	}
+}
+# END CODE Copyright (C) 2012 by Andrew Johnson.
+
+sub debug { my $self = shift; @_ ? $self->{debug} = shift : $self->{debug}; }
+
 # these are for direct access to the underlying TheSchwartz::Job object
 sub job { my $self = shift; @_ ? $self->{job} = shift : $self->{job}; }
 
-
 sub setArgXML { $_[0]->{argxml} = $_[1]; }
 sub getArgXML { return $_[0]->{argxml}; }
 
-sub debug { my $self = shift; @_ ? $self->{debug} = shift : $self->{debug}; }
-
 
 =head1 METHODS
 
@@ -287,7 +336,7 @@ sub failed {
             }
         };
 	}	
-	$job->failed($error, $exitstatus);
+	$job->failed(substr($error,0,254), $exitstatus);
 	return $exitstatus;
 }
 
@@ -344,10 +393,35 @@ sub failedNoRetry {
         };
 	}
 
-	$job->permanent_failure($error, $exitstatus);
+	$job->permanent_failure(substr($error,0,254), $exitstatus);
 	return $exitstatus;
 }
 
+=head2 deferred()
+
+Defers processing of a job even though it was available for processing in the 
+queue.  The job will be seen as available for processing again when the 
+grabbed_until time has expired (the default is 60 minutes).  If your service 
+employs the job retry API, a declined job run does not count against the job's 
+retry count.
+
+Unlike the completed() and failed*() methods above, deferred() is actually 
+only a wrapper around TheSchwartz 1.10's TheSchwartz::Job->declined() method 
+for now.  No job history is recorded in the HELIOS_JOB_HISTORY_TB in the 
+collective database.  This may change in the future.
+
+=cut
+
+sub deferred {
+	my $self = shift;
+	my $job = $self->job();
+
+	$job->declined();
+	return 0;
+}
+
+
+=head1 JOB SUBMISSION
 
 =head2 submit()
 
@@ -356,11 +430,71 @@ error if it fails.
 
 Before a job can be successfully submitted, the following must be set first:
 
- $job->setConfig()
- $job->setArgXML()
- $job->setFuncname()
+ $job->setConfig($configHash);
+ $job->setArgXML($xmlstring);
+ $job->setFuncname($servicename);
 
+So, for example, to submit a Helios::TestService to the Helios system, you need 
+to do the following:
 
+ # you need Helios::Service and Helios::Job
+ use Helios::Service;
+ use Helios::Job;
+
+ # these are the job arguments we want to pass to Helios::TestService
+ my $jobxml = "<job><params><string1>This is a test</string1/params>/job>";
+
+ # first, use Helios::Service to get the Helios configuration
+ my $srv = Helios::Service->new();
+ $srv->prep();
+ my $config = $srv->getConfig();
+ 
+ # once you have the config, you can set up the Helios::Job
+ my $job = Helios::Job->new();
+ $job->setConfig($config);
+ $job->setFuncname('Helios::TestService');
+ $job->setArgXML();
+ 
+ # then submit the job (this will throw an exception if something goes wrong)
+ my $jobid = $job->submit();
+ print "Submitted job $jobid to Helios\n";
+ 
+Both Helios::Service->prep() and Helios::Job->submit() will throw exceptions 
+if they encounter errors, so a safer example would catch them:
+
+ use Helios::Service;
+ use Helios::Job;
+
+ my $srv = Helios::Service->new();
+ eval {
+ 	$srv->prep();
+ 	1;
+ } or do {
+ 	my $E = $@;
+ 	print "Error encountered prepping Helios service: $E\n";
+ 	exit(1);
+ };
+ my $config = $srv->getConfig();
+ 
+ # once you have the config, you can set up the Helios::Job
+ my $job = Helios::Job->new();
+ $job->setConfig($config);
+ $job->setFuncname('Helios::TestService');
+ $job->setArgXML();
+ 
+ # then submit the job (this will throw an exception if something goes wrong)
+ my $jobid;
+ eval {
+	$jobid = $job->submit();
+ 	1;
+ } or do {
+	my $E = $@;
+	print "Error encountered attempting job submission: $E\n"; 	
+ };
+ print "Submitted job $jobid to Helios\n";
+
+Of course, the Try::Tiny (available on CPAN) would work just as well as an
+eval{} block, and have much prettier syntax.
 
 =cut
 
@@ -379,7 +513,7 @@ sub submit {
 
 	my $args = [ $params ];
 
-	my $client = TheSchwartz->new( databases => $databases, verbose => 1 );
+	my $client = Helios::TheSchwartz->new( databases => $databases, verbose => 1 );
 	my $sjh = $client->insert($job_class, $args);
 	$self->setJobid($sjh->jobid);
 	return $sjh->jobid;
@@ -455,26 +589,28 @@ sub burst {
 
 =head1 OTHER METHODS
 
-=head2 getDriver()
+=head2 initDriver()
 
 Returns a Data::ObjectDriver object for use with Helios layer database updates.
 
 =cut
 
-sub getDriver {
+# BEGIN CODE Copyright (C) 2012 by Andrew Johnson.
+
+sub initDriver {
 	my $self = shift;
 	my $config = $self->getConfig();
 	if ($self->debug) { print $config->{dsn},$config->{user},$config->{password},"\n"; }
-	my $driver = Data::ObjectDriver::Driver::DBI->new(
+	my $driver = Helios::ObjectDriver::DBI->new(
 	    dsn      => $config->{dsn},
 	    username => $config->{user},
 	    password => $config->{password}
 	);	
-	if ($self->debug) { print "DRIVER: ",$driver,"\n"; }
+	if ($self->debug) { print 'Job->initDriver() DRIVER: ',$driver,"\n"; }
+	$self->setDriver($driver);
 	return $driver;	
 }
-
-
+# END CODE Copyright (C) 2012 by Andrew Johnson.
 
 
 1;
@@ -483,16 +619,19 @@ __END__
 
 =head1 SEE ALSO
 
-L<Helios::Service>, L<Helios::Error>, L<TheSchwartz::Job>, L<XML::Simple>, L<Config::IniFiles>
+L<Helios>, L<Helios::Service>, L<TheSchwartz::Job>, L<XML::Simple>
 
 =head1 AUTHOR
 
-Andrew Johnson, E<lt>ajohnson@ittoolbox.comE<gt>
+Andrew Johnson, E<lt>lajandy at cpan dotorgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2008 by CEB Toolbox, Inc.
 
+Portions of this software, where noted, are
+Copyright (C) 2012 by Andrew Johnson.
+
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.0 or,
 at your option, any later version of Perl 5 you may have available.
